﻿// Paxnaman, a new clone of the old classic game Puckman.
// Copyright (C) 2009  Gorka Suárez García
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

namespace Pacman {
    /// <summary>
    /// This class represents an animated Puckman entity with a sequence.
    /// </summary>
    public sealed class SequencedPuckman : AnimatedPuckman {
        //******************************************************************************************
        // Data
        //******************************************************************************************

        #region Sequence Sequence
        /// <summary>
        /// The sequence of the entity.
        /// </summary>
        public Sequence Sequence { get; set; }
        #endregion

        //******************************************************************************************
        // Constructors
        //******************************************************************************************

        #region SequencedPuckman()
        /// <summary>
        /// Constructs a new SequencedPuckman object.
        /// </summary>
        /// <param name="sequence">The sequence of the entity.</param>
        public SequencedPuckman(Sequence sequence) {
            this.Sequence = sequence;
        }
        #endregion

        //******************************************************************************************
        // Methods
        //******************************************************************************************

        #region void Update(long time)
        /// <summary>
        /// Updates the inner state of the entity.
        /// </summary>
        /// <param name="time">The passed time after the last call of the update.</param>
        public override void Update(long time) {
            if(this.Alive) {
                // When the entity is alive, we'll control the tick logic of the entity.
                this.currentTime += time;
                while(this.currentTime >= this.maxTimeInterval) {
                    this.currentTime -= this.maxTimeInterval;

                    // In every tick, we'll check the sequence and the direction first.
                    this.Sequence.Check(this);
                    this.testNeededDirection();

                    // And finally, we'll update the coordinates and the sprites.
                    if(this.updateCoordinates()) {
                        this.sprites[this.currentSprite].Update(this.maxTimeInterval);
                    }
                }
            } else {
                // When the entity isn't alive, we only update the sprites.
                this.sprites[this.currentSprite].Update(time);
            }
        }
        #endregion
    }
}